      program sids8
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cc
c   name - sids8                              module - d.5.3
c   language - ftn v                          type - main program 
c   version - 3.6         date - 06/05/84     programmer -
c                                               harris, l.j. - prc kentron
c                                               knight, g.e. - prc kentron
c   version - 3.7         date - 04/03/86       johnson, m.j. - prc kentron 
c   version - 3.8         date - 10/11/88     programmer -
c                                               johnson, m.j. - prc kentron 
c                                               terray, b.    - prc kentron 
c                         date - 07/05/89       flug, b.t.    - prc 
c   version - 3.9         date - 06/08/90       flug, b.t. - lockheed 
c   version - 4.0	  date - 01/15/93	gupta, s. - lockheed
c
c purpose
c	to read and dump selected records from the (S-8)
c
c input files
c
c	tape8	- contains PAT data
c
c local variables
c
c	cff	- form feed character for the printer
c	ibegrc	- beginning record to be dumped
c	idata	- physical header
c	iendrc	- end record to be dumped
c	iertyp	- type of I/O error
c	ios	- I/O error number
c	ipat	- integer PAT array & temporary holding place for scale
c		  factors and offsets until they are converted
c	irc	- byte counter
c	irec	- record counter
c	is8out	- unit number for S-8 data file
c	iunout	- unit number for output file
c	off	- real offsets' array
c	parmvl	- character value representing passed value
c	pat	- real PAT array
c	prec	- PAT data from file
c	s8name	- name of PAT file
c	sf	- real scale factors' array
c	itrec	- total number of records dumped
c
c subroutines called
c
c	conversion	- convert data from integer to real
c	dmprecs	- write out up to 10 PAT records
c	getarg	- get parameter from execution control statement
c       ppspec	- print product specifications
c	prphed	- print physical header
c	sysmsg	- handle sybsystem error message processing
c	unpack	- unpack the S-8 data
c	unscale	- apply the scale factors and offsets to the data
c
c output files
c	tape9	- contains formatted output
c
c configuration control summary - 
c      1.  version 3.8, 09/30/88. 
c          program was modified to enhance labeling of output 
c          [669]. 
c 
c      2.  version 3.9, 06/08/90. 
c          program was modified to include scanner offset file flag 
c          [866]. 
c
c      3.  version 4.0, 01/15/93.
c          Program was modified to run under UNIX operating system.  Sun
c	   utility getarg is used to retrieve arguments from execution 
c	   control statement.  Record structure /patrec/ contains a PAT
c	   record with 3570 values and is used to almost eliminate the need 
c	   for packing and unpacking the data.  The 4-bit values still need 
c	   to be unpacked.  The structure consists of 15-element integer*4 
c	   array, m32, 3225- element integer*2 array, m16, and 330-element
c	   byte array, m8.
c
c comments
cc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      structure /patrec/
	integer * 4 m32(15)
	integer * 2 m16(3225)
	byte m8(330)
      end structure
c
      record /patrec/ prec
      character * 1 cff
      character * 9 parmvl
      character * 20 s8name
      character * 32 label
      integer * 2 idata(15)
      dimension ipat(3630), off(3630), pat(10, 3630), sf(3630)
      dimension index(67), label(67)
c
      data is8out, iunout, idatlbl /8, 9, 100/
      cff = char(12)
c
c get the name of the S-8 data file
      call getarg(1, s8name)
c
c open the S-8 data file
      open(is8out, file = s8name, access = 'direct', form = 
     /'unformatted', recl = 1, iostat = ios)
      if (ios .ne. 0) then
         iertyp = 1
         call sysmsg(ios, iertyp, s8name)
      end if
c
c open the label file
      open(idatlbl, file = 'patrcfm', form = 'formatted', iostat
     /= ios)
      if (ios .ne. 0) then
         iertyp = 1
         call sysmsg(ios, iertyp, 'patrcfm')
      end if
c
c read the header
      irc = 1
      read(is8out, rec = irc, iostat = ios) idata
      irc = irc + 30
      if (ios .ne. 0) then
         iertyp = 2
         call sysmsg(ios, iertyp, s8name)
      end if
c
c print the product specifications
      call ppspec(idata, s8name, iunout)
      write(iunout, '(a1)') cff
c
c print the header
      call prphed(iunout, idata)
      write(iunout, '(a1)') cff
c
c print the labels
      read(idatlbl, '(i4, 5x, a32)', iostat = ios) (index(i), label(i), 
     /i = 1, 67)
      if (ios .ne. 0) then
         iertyp = 2
         call sysmsg(ios, iertyp, 'patrcfm')
      end if
      do 50 i = 1, 34
         if (i .lt. 34) then
            write(iunout, 2010) (index(j), label(j), j = i, i + 34, 34)
2010        format(8x, i4, 7x, a32, 7x, i4, 7x, a32)
         else
            write(iunout, 2015) index(i), label(i)
2015        format(8x, i4, 7x, a32)
         end if
50    continue
c
c get the start and end records passed as parameters
      call getarg(2, parmvl)
      read(parmvl, '(i4)', iostat = ios) ibegrc
      if (ios .ne. 0) then
         iertyp = 2
         call sysmsg(ios, iertyp, 'internal')
      end if
      call getarg(3, parmvl)
      read(parmvl, '(i4)', iostat = ios) iendrc
      if (ios .ne. 0) then
         iertyp = 2
         call sysmsg(ios, iertyp, 'internal')
      end if
      itrec = iendrc - ibegrc + 1
c
c terminate if only the header is dumped
      if (ibegrc .eq. 0) goto 900
c
c skip the test record
      irc = irc + 6840
c
c read the scale factors
      read(is8out, rec = irc, iostat = ios) prec
      irc = irc + 6840
      if (ios .ne. 0) then
         iertyp = 2
         call sysmsg(ios, iertyp, s8name)
      end if
c
c assign and unpack the scale factors
      call unpack(prec, ipat)
      call conversion(ipat, sf)
c
c read the offsets
      read(is8out, rec = irc, iostat = ios) prec
      irc = irc + 6840
      if (ios .ne. 0) then
         iertyp = 2
         call sysmsg(ios, iertyp, s8name)
      end if
c
c assign and unpack the offsets
      call unpack(prec, ipat)
      call conversion(ipat, off)
c
c read the pat record
      irc = 20551 + (ibegrc - 1) * 6840
      irec = 1
      k = 1
100   read(is8out, rec = irc, end = 900, iostat = ios) prec
      if (ios .ne. 0) then
         iertyp = 2
         call sysmsg(ios, iertyp, s8name)
      end if
c
c assign and unpack the record
      call unpack(prec, ipat)
c
c unscale the data
      call unscale(ipat, k, sf, off, pat)
c
      if ((k .lt. 10) .and. (irec .lt. itrec)) then
         k = k + 1
         irec = irec + 1
         irc = irc + 6840
         goto 100
      end if
      if ((k .eq. 10) .or. (irec .eq. itrec)) then
         write(iunout, '(a1)') cff
         write(iunout, 1020) (irec - k + m, m = 1, k)
1020     format(//1x, 13hRecord Num =, 10x, i4, 9(8x, i4))
         write(iunout, *)
         n = 5
         call dmprecs(k, pat, n, irec, iunout, idatlbl)
         if (irec. lt. itrec) then
            k = 0
            goto 100
         end if
      end if
900   stop
      end
c
      subroutine conversion(iarr, arr)
c   ***************************************************************
c
c name	- conversion
c language	- ftn v
c version	- 1.0		date	- 1/15/1993	programmer-
c							   gupta, s.
c
c purpose
c	to convert integers to reals
c
c input parameters
c	iarr	- integer array
c
c output parameters
c	arr	- real array
c
c local parameters
c	i	- counter
c
      dimension arr(3630), iarr(3630)
c
      do 50 i = 1, 3630
         arr(i) = float(iarr(i))
50    continue
c
      return
      end
c
       subroutine datchr(wjd,fjd, ireslt)
c   **********************************************************
c 
c   name - datchr                             module -
c   language - ftn v                          type - subroutine 
c   version - 1.0         date - 12/18/92     programmer -
c                                               nolan, s.k. - lesc
c             1.1                01/05/93       gupta, s.   - lesc
c   
c   purpose - to create a character string  containing the
c             time span of a given day based on a julian
c             date.
c
c   input parameters  - 
c      wjd     - whole part of julian date 
c      fjd     - fractional part of julian date 
c
c   output parameters  - 
c      ireslt  - character string containing the time
c                span of month corresponding to julian date
c
c   key local parameters  -
c      iyear, imon  - integer forms of the date calculated by julcal
c      lday  - integer form of day of the month
c      xdate - the calender date and time calculated by julcal
c   **********************************************************
       dimension  xdate(6)
       character*30 ireslt
c
       ireslt = '  /0 /0  0000 -   /0 /0  2359 ' 
       call julcal(wjd,fjd,0,xdate)
       iyear = int(xdate(1))
       imon = int(xdate(2))
       lday = int(xdate(3))
       write(ireslt(1:2),'(i2)')iyear
       write(ireslt(17:18),'(i2)')iyear
       if (imon .lt. 10)then
            write(ireslt(5:5),'(i1)')imon
            write(ireslt(21:21),'(i1)')imon
       else
            write(ireslt(4:5),'(i2)')imon
            write(ireslt(20:21),'(i2)')imon
       endif
       if (lday .lt. 10) then
          write(ireslt(8:8), '(i1)') lday
          write(ireslt(24:24),'(i1)')lday
       else
          write(ireslt(7:8), '(i2)') lday
          write(ireslt(23:24), '(i2)') lday
       endif
c
       return
       end
c
      subroutine dmprecs
     i                  ( k, xpat, n, irec, iunout, idatlbl ) 
c 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c 
c   name - dmprecs                            module -
c   language - ftn v                          type - subroutine 
c   version - 3.0         date - 08/12/88     programmer -
c                                               terray, b.a. - prc
c 
c 
c   purpose - 
c     print out up to 10 recs from the id-4, id-24, id-25 or the s-8
c 
c   input parameters -
c      idatlbl - file containing record index, no. of characters in the 
c                labels, labels, and no. of lines of data for each label
c      irec    - record counter 
c      iunout  - unit number associated with standard output
c      k       - number of records to be dumped 
c      n       - counts the number of lines of information printed on 
c                a page 
c      xpat    - array containing up to 10 pat records
c 
c   key local parameters -
c      afile   - file containing the value of chars 
c      cff     - end of page character for the printer
c      chars   - has a value of [scan no., pixel no.] for scanner data
c                [measurement number] for nonscanner data, or blank 
c                for other data 
c      contval - indicates if data is continued from previous page
c      geoscns - geo-scene identifications
c      index   - value of the record index for current label
c      istrln  - number of characters in the label
c      label   - current output label 
c      lbls    - counter for the labels 
c      nline   - number of lines of data to be printed under the label
c      scntyps - scene type identifications 
c      signns  - array containing sign associated with nonscanner record
c                level flags
c      signs   - array containing sign associated with scanner record 
c                level flags
c 
c   subroutines called -
c     none
c 
c   exit states - 
c     normal return 
c 
c   comments -
c 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c 
      character *1 signs( 10), signns( 10), cff
      character *6 afile, chars 
      character *13 contval 
      character *32 label 
      character geoscns*47, scntyps*93
c 
      dimension xpat(10, 3630)
c 
      data geoscns / '( x.0-ocn, x.1-lnd, x.2-sno, x.3-des, x.4-mix )' /
      data scntyps / '(1-ocn,2-lnd,3-sno,4-des,5-mix,6-pc/ocn,7-pc/lnd,8
     1-pc/mix,9-mc/ocn,10-mc/lnd,11-mc/mix,12-ov)' /
c
      cff = char(12)
c 
      do 100 l = 1, k 
c 
          signs( l) = ' ' 
          signns( l) = ' '
c 
          if ( xpat(l, 2135) .lt. 0.0 ) then
c 
             xpat(l, 2135) = - xpat(l, 2135) 
             signs( l) = '-'
c 
          end if
c 
          if ( xpat(l, 2137) .lt. 0.0) then 
c 
             xpat(l, 2137) = - xpat(l, 2137) 
             signns( l) = '-' 
c 
          end if
c 
  100 continue
c 
c *** end do
c 
c 
c *** initialize counter for printing.
c 
      j = 0 
c 
      rewind (idatlbl)
c 
c *** print out data, including labels. 
c 
      do 300 lbls = 1, 67 
c 
         read(idatlbl,'(2i4,1x,a32,i4)', iostat = ios) index, 
     /   istrln, label, nline
         if (ios .ne. 0) then
            iertyp = 2
            call sysmsg(ios, iertyp, 'patrcfm')
         end if
c 
c *** print out at top of new page if page is full. 
c 
         if( ( n  .ge. 44 ) .or. ( nline .ge. 40 ) ) then 
c 
          write(iunout, '(a1)') cff
          write(iunout,1020) ( irec - k + m, m = 1, k ) 
          write(iunout,*) 
          n = 5 
c 
         end if 
c 
         contval = '             '
         chars = '      ' 
         l = 1
         icnt = 0 
c 
c *** print out data label. 
c 
         do 200 line = 1, nline 
c 
            icnt = icnt + 1 
            j = j + 1 
c 
c *** if printing scanner data, get [scan no., pixel no.].
c 
            if( nline .eq. 248 ) then 
c 
             if(icnt .eq. 63 .or. icnt .eq. 125 .or. icnt .eq. 187) then
                l = l + 1 
             end if 
c 
             write(afile,3000) l, icnt - (l - 1)*62 
c 
             read(afile,'(a6)', iostat = ios) chars 
             if (ios .ne. 0) then
                iertyp = 2
                call sysmsg(ios, iertyp, 'internal')
             end if
c 
c *** if printing nonscanner data, get [measurement number].
c 
            else if( nline .eq. 20 ) then 
c 
             write(afile,3010) icnt 
c 
             read(afile,'(a6)', iostat = ios) chars 
             if (ios .ne. 0) then
                iertyp = 2
                call sysmsg(ios, iertyp, 'internal')
             end if
c 
            end if
c 
c *** print out at top of new page if page is full. 
c 
            if( n .eq. 47 ) then
c 
                write(iunout,'(a1)') cff
                write(iunout,1020) ( irec - k + m, m = 1, k ) 
                n = 4 
c 
            end if
c 
c *** print out data label. 
c 
            if( ( line .eq. 1 ) .or. ( n .eq. 4 ) ) then
c 
c *** if data is continued from previous page, include '(continued)'
c     after label.
c 
              if( line .ne. 1 ) contval = ' (continued) ' 
c 
c 
              if( lbls .eq. 62 ) then 
c 
                write(iunout,1010) label(1:istrln)//scntyps// 
     1                             contval
                write(iunout,'(25x,a47)') geoscns 
                n = n + 3 
c 
              else
c 
                if( nline .eq. 248 ) then 
c 
                    write(iunout,1010) label(1:istrln)//
     1                    ' [scan number,pixel number]'//contval
c 
                else if( nline .eq. 20 ) then 
c 
                  if( label(1:4) .eq.'flag' ) then
                    write(iunout,1010) label(1:istrln)//
     1                   ' (octal)'//' [measurement number]'//contval 
                  else
                    write(iunout,1010) label(1:istrln)//
     1                    ' [measurement number]'//contval
                  end if
c 
                else if( label(1:4) .eq. 'flag' .and. 
     1                   nline .ne. 20 ) then 
c 
                    write(iunout,1010) label(1:istrln)//
     1                                 ' (octal)'//contval
c 
                else
c 
                    write(iunout,1010) label(1:istrln)//contval 
c 
                end if
c 
                n = n + 2 
c 
              end if
c 
            end if
c 
c *** print out the records.
c 
         n = n + 1
c 
         if ( ( j .eq. 2 ) .or. ( j .eq. 3 ) ) then 
             write ( iunout, 1030) j, chars, ( xpat(m, j), m = 1, k) 
c 
         else if ( ( j .eq. 1 ) .or. ( j .le. 2134 ) ) then 
c 
             write ( iunout, 1040) j, chars, ( xpat(m, j), m = 1, k) 
c 
         else if ( j .eq. 2135 ) then 
c 
             write ( iunout, 1060) j, chars,
     1             ( signs( m), int( xpat( m, j) ), m = 1, k) 
c 
         else if ( j .eq. 2136 ) then 
c 
             write ( iunout, 1050) j, chars,
     1             ( int( xpat(m, j) ) , m = 1, k) 
c 
         else if ( j .eq. 2137 ) then 
c 
             write ( iunout, 1060) j, chars,
     1             ( signns( m), int( xpat(m, j) ), m = 1, k)
c 
         else if ( j .ge. 2138 .and. j .le. 2220 ) then 
c 
             write ( iunout, 1050) j, chars,
     1             ( int( xpat(m, j) ) , m = 1, k) 
c 
         else if ( j .ge. 2221 .and. j .le. 3488 ) then 
c 
             write ( iunout, 1040 ) j, chars, ( xpat(m, j), m = 1, k)
c 
         else if ( j .eq. 3489 ) then 
c 
             write ( iunout, 1050) j, chars,
     1             ( int( xpat(m, j) ) , m = 1, k) 
c 
         else if ( j .ge. 3490 .and. j .le. 3510 ) then 
c 
             write ( iunout, 1040) j, chars, ( xpat(m, j), m = 1, k)
c 
         else if ( j .ge. 3511 .and. j .le. 3550 ) then 
c 
             write ( iunout, 1050) j, chars,
     1             ( int( xpat(m, j) ) , m = 1, k) 
c 
         else if ( j .ge. 3551 .and. j .le. 3630 ) then 
c 
             write ( iunout, 1040) j, chars, ( xpat(m, j ), m = 1, k) 
c 
         end if 
c 
  200    continue 
c 
c *** end do
c 
  300 continue
c 
c *** end do
c 
      return
c 
c *** format statements.
c 
 1010 format( /, 1x, a )
 1020 format ( //1x, 13hrecord num = , 10x, i4, 9(8x, i4))
 1030 format ( 1x, 5hxpat(, i4, 1h), a6, f11.7, 9( 1x,f11.7)) 
 1040 format ( 1x, 5hxpat(, i4, 1h), a6, f11.2, 9( 1x, f11.2))
 1050 format ( 1x, 5hxpat(, i4, 1h), a6, 1x, o10, 9( 2x, o10))
 1060 format ( 1x, 5hxpat(, i4, 1h), a6, a1, o10, 9( 1x, a1, o10))
 3000 format ( 1h[, i1, 1h,, i2, 1h] )
 3010 format ( 1h[, i2, 1h] ) 
c
      end 
c
      subroutine julcal 
     i                (wholjd, fracjd, ind, 
     b
     o                 dat) 
c 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c 
c  name - julcal                                 module - g.e.8.5.2 
c  language - ftn v                              type - utility 
c  version - release 3.0     date - 08/03/82     programmer - zimmerman 
c 
c  purpose -
c     to convert a julian date to the corresponding calendar date 
c 
c  input parameters - 
c     wholjd : whole number part of the julian date (real)
c     fracjd : fractional part of the julian date (real decimal)
c        ind : control variable set to 0 to indicate input as julian date;
c              set to 1 to indicate input as number of days elapsed since 
c              0000 hours 01 january 1950 (integer) 
c 
c  output parameters -
c     dat : array of date elements as 2-digit real
c        dat(1)=yr
c        dat(2)=mo
c        dat(3)=da
c        dat(4)=hr
c        dat(5)=min 
c        dat(6)=sec     (with fractional part)
c 
c  key local parameters - 
c     intday : holds whole days since 0000 hr 01 january 1949 
c     fraday : holds portion of day since same
c     leapcy : number of four-year periods (1461 days)
c     morday : number of excess days after accounting for years 
c     iyr49  : number of years elapsed since 01 january 1949
c     moryr  : number of excess years after accounting for leapcy 
c     ymd    : yymm.dd and yy.mmdd
c     iyrmo  : yymm 
c     isec   : number of seconds in fractional part 
c 
c  subroutines called - 
c     local subroutine    - yymmdd (g.e.8.5.4)
c     intrinsic function  - int    (ftn v)
c     intrinsic function  - nint   (ftn v)
c     intrinsic function  - mod    (ftn v) 
c 
c  comments - 
c     the algorithm for this sub was provided by ed howerton.  the
c     nint function used in this sub is used to avoid truncation
c     errors in computations.  the general approach is to calculate 
c     days since 0000 hr on 01 january 1949.
c 
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
c 
c*****
      parameter ( ianal = 1 ) 
c 
c$    if ( ianal .eq. 1 ) then
         common /utlcnt/ ncalls(60) 
c$    endif 
c*****
      dimension dat(6)
c 
c$    if ( ianal .eq. 1 ) then
         ncalls( 23 ) = ncalls( 23 ) + 1
c$    endif 
c 
c *** the program does not combine whole and fractional parts, but
c *** works with each part separately.  this is done to maintain the
c *** maximum number of significant digits for each value.
c 
      if ( ind .eq. 0 ) then
         if ( fracjd .ge. 0.5 ) then
            intday = nint ( wholjd - 2432917. ) 
            fraday = fracjd - 0.5 
         else 
            fraday = fracjd + 0.5 
            intday = nint ( wholjd - 2432918. ) 
         end if 
      else
         intday = nint ( wholjd + 365. )
         fraday = fracjd
      end if
c 
      leapcy = intday / 1461
      morday = intday - ( leapcy * 1461 ) 
      iyr49  = leapcy * 4 
c 
c *** if the elapsed time is a multiple number of 4-year cycles with
c *** less than one year of additional time, these values are now 
c *** valid.  if a year or more remains in addition, however, the number
c *** of years must be accounted for.  because leap year is the fourth
c *** year of each 4-year cycle since 01 january 1949, each whole 
c *** elapsed year is 365 days in length. 
c 
      if ( morday .ge.365 ) then
         moryr  = morday / 365
         if (moryr .eq. 4) moryr = 3
         iyr49  = iyr49  + moryr
         morday = morday - ( moryr * 365 )
      end if
c 
      iyr  = iyr49 + 49 
      dat(1) = iyr
c 
c *** morday indicates how many additional days into the year must be 
c *** accounted for.  if it is the first day of the year, no additional 
c *** days need be accounted for, because 01 january is an exact number 
c *** of elapsed years.  on january 2, 1 more day must be accounted for.
c *** in general, to account for x more days, we need the x+1 day of
c *** the year. 
c 
      idayyr = morday + 1 
c 
c *** the function yymmdd will return the day of the year based on
c *** a number of days elapsed. 
c 
      call yymmdd 
     i          (iyr, idayyr, 
     b
     o           iymd)
c 
c *** from iymd, the month and day must be separated. 
c
      dat(3) = float(mod(iymd,100)) 
      iyrmo  = iymd/100 
      dat(2) = float(mod(iyrmo,100))
c 
c *** the fractional part of the day will be converted into seconds by
c *** multiplying by total seconds per day (86400).  the excess will
c *** be truncated from seconds and captured into dat(6). 
c 
      isec   = int( 86400. * fraday ) 
      dat(6) = 86400. * fraday - float(isec)
      ihr    = isec / 3600
      dat(4) = float( ihr ) 
      isec   = isec - (ihr * 3600)
      imin   = isec / 60
      dat(5) = float( imin )
      dat(6) = float( isec - (imin *60 )) + dat(6)
c 
      return
      end 
c
      subroutine ppspec(idata, s8name, iunout)
c   ***************************************************************
c
c name	- ppspec
c language	- ftn v
c version	- 3.9		date	- 1/15/1992	programmer-
c							   gupta, s.
c
c purpose
c	to print the product specifications
c
c input parameters
c	idata	- physical header
c	iunout	- unit number for output
c	s8name	- file name
c
c local parameters
c	fjd	- fractional julian date
c	ireslt	- string containing temporal span
c	wjd	- whole julian date
c
      integer * 2 idata
      dimension idata(15)
      character * 9 s8name
      character * 30 ireslt
c
      write(iunout, 5)
5     format(1x, 'Product Specifications:', /)
      wjd = float(idata(4) * 10000) + float(idata(5))
      fjd = float(idata(6)) / 10000
      call datchr(wjd, fjd, ireslt)
      write(iunout, 10) ireslt
10    format(4x, 'Temporal Span: ', a30, /)
      if (idata(3) .eq. 1) then
         write(iunout, 20)
20       format(4x, 'Spacecraft: NOAA F', /)
      else if (idata(3) .eq. 2) then
         write(iunout, 30)
30       format(4x, 'Spacecraft: ERBS', /)
      else if (idata(3) .eq. 3) then
         write(iunout, 40)
40       format(4x, 'Spacecraft: NOAA G', /)
      end if
      write(iunout, 50)
50    format(4x, 'Product: S-8', /)
      write(iunout, 60) s8name
60    format(4x, 'VSN/file: ', a9, /)
c
      return
      end
c
      subroutine prphed 
     i                 (iunout, idata )
c 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c 
c   name - prphed                             module - g.5.21 
c   language - ftn v                          type - subroutine 
c   version - 3.6         date - 04/03/86     programmer -
c                                               johnson, m.j. - prc kentron 
c 
c 
c   purpose - 
c     to write an external output form physical header record
c 
c   input parameters - 
c     iunout - unit number for printed output 
c     idata  - array containing unpacked physical header information
c 
c   output files - 
c 
c   output parameters - 
c     none
c
c   subroutines called -
c     none
c 
c   exit states - 
c     normal return 
c 
c   restrictions -
c 
c   attributes -
c 
c   comments -
c 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c 
c 
      integer*2  idata( 15) 
c 
c ***  print out contents of header 
c 
       write ( iunout, 1000) ( idata( i), i = 1, 13 )
c 
c *** format statements 
c 
 1000 format (/, 5x, 28hphysical header contents:    // 
     1           5x, 28hsubsystem indicator       : , i10 / 
     2           5x, 28hproduct code              : , i10 / 
     3           5x, 28hspacecraft indicator      : , i10 / 
     4           5x, 28hwhole julian date - high  : , i10 / 
     5           5x, 28hwhole julian date - low   : , i10 / 
     6           5x, 28hfractional julian date    : , i10 / 
     7           5x, 28hprocessed version counter : , i10 / 
     8           5x, 28hyear processed            : , i10 / 
     9           5x, 28hmonth processed           : , i10 / 
     a           5x, 28hday processed             : , i10 / 
     b           5x, 28hhour processed            : , i10 / 
     c           5x, 28hminute processed          : , i10 / 
     d           5x, 28hsecond processed          : , i10 /// ) 
c 
      return
c 
      end 
      subroutine sysmsg ( i, j, filename )
c 
c 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c 
c   name - sysmsg                             module -
c   language - ftn v                          type - subroutine 
c   version -             date - 87/01/13     programmer -
c                                               johnson, m.j. - prc kentron 
c 
c 
c   purpose - to write error messages to an error message file. 
c 
c   input files - 
c       none
c 
c   input parameters -
c       i      - error number
c       j      - error classification code; a code of 1 
c                indicates an error in opening a file; a code of 2 indicates
c                an error in reading from the file 
c       filename - name of the file where I/O error was trapped
c
c   output files -
c       tape16 - error message file 
c 
c   output parameters - 
c         none
c 
c   subroutines called -
c         none
c 
c   exit states - 
c         none
c 
c   restrictions -
c 
c   attributes -
c 
c   comments -
c 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c 
       character * 9 filename
c 
       if (j.eq.1) then
	  write(16, 10) i, filename
10	  format(1x, 'I/O error number', i4, ' occurred while opening
     / the file ', a9)
	else if (j.eq.2) then
	  write(16, 20) i, filename
20	  format(1x, 'I/O error number', i4, ' occurred while reading
     / from the file ', a9)
      endif
c
	stop
	end
c
      subroutine unpack(prec, iarr)
c   ****************************************************************
c name	- unpack
c language	- ftn v
c version	- 1.0		date	- 1/15/1992	programmer-
c							   gupta, s.
c
c
c input parameters
c	prec	- PAT data from file
c
c output parameters
c	iarr	- unpacked array
c
c local parameters
c	i	- counter
c	j	- number of 4-bit values
c	m4	- 4-bit PAT values
c
      structure /patrec/
         integer * 4 m32(15)
         integer * 2 m16(3225)
         byte m8(330)
      end structure
c
      record /patrec/ prec     
      byte m4(120)
      integer two31, two32, two15, two16
      dimension iarr(3630)
c
      two31 = 2 ** 31
      two32 = 2 ** 32
      two15 = 2 ** 15
      two16 = 2 ** 16
c
      do 50 i = 1, 15
         if (prec.m32(i) .gt. two31) then
            iarr(i) = prec.m32(i) - two32
         else
            iarr(i) = prec.m32(i)
         end if
50    continue
c
      do 100 i = 1, 3225
         if (prec.m16(i) .gt. two15) then
            iarr(i + 15) = prec.m16(i) - two16
         else
            iarr(i + 15) = prec.m16(i)
         end if
100   continue
c
      do 150 i = 1, 270
         iarr(i + 3240) = prec.m8(i)
150   continue
c
      j = 1
      do 200 i = 271, 330
         if (prec.m8(i) .eq. -1) then
            m4(j) = 15
            m4(j + 1) = 15
         else
            m4(j) = int(prec.m8(i) / 16)
            m4(j + 1) = mod(prec.m8(i), 16)
         end if
         j = j + 2
200   continue
c
      do 250 i = 1, j - 1
         iarr(i + 3510) = m4(i)
250   continue
c
      return
      end
c
      subroutine unscale(iarr, k, sf, off, arr)
c  ****************************************************************
c name	- unscale
c language	- ftn v
c version	- 1.0		date	- 1/15/1993	programmer-
c							   gupta, s.
c
c
c purpose
c	to unpack the PAT data
c
c input parameters
c	iarr	- integer PAT array
c	off	- offset array
c	sf	- scale factor array
c
c output parameters
c	arr	- the actual PAT data
c
c local parameters
c	i	- counter
c
      integer deflt32, deflt16, deflt8, deflt4
      dimension arr(10, 3630), iarr(3630), sf(3630), off(3630)
c
      deflt32 = 2 ** 31 - 1
      deflt16 = 2 ** 15 - 1
      deflt8 = 2 ** 7 - 1
      deflt4 = 2 ** 4 - 1
c 
      do 50 i = 1, 15
         if (iarr(i) .eq. deflt32) then
            arr(k, i) = float(deflt16)
         else
            arr(k, i) = float(iarr(i)) / sf(i) - off(i)
         end if
50    continue
c
      do 100 i = 16, 3240
         if (iarr(i) .eq. deflt16) then
            arr(k, i) = float(deflt16)
         else
            arr(k, i) = float(iarr(i)) / sf(i) - off(i)
         end if
100   continue
c
      do 150 i = 3241, 3510
         if (iarr(i) .eq. deflt8) then
            arr(k, i) = float(deflt16)
         else
            arr(k, i) = float(iarr(i)) / sf(i) - off(i)
         end if
150   continue
c
      do 200 i = 3511, 3630
         if (iarr(i) .eq. deflt4) then
            arr(k, i) = float(deflt16)
         else
            arr(k, i) = float(iarr(i)) / sf(i) - off(i)
         end if
200   continue
c
      return
      end
c
      subroutine yymmdd 
     i                (iyr, iday, 
     b
     o                 iyymmd)
c 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c 
c  name - yymmdd                                 module - g.e.8.5.4 
c  language - ftn v                              type - subroutine
c  version - 3.0             date - 08/03/82     programmer - tolson
c                                                          zimmerman
c 
c  purpose -
c     change from number of day of the year to a date 
c 
c  input parameters - 
c     iyr  : 2-digit year (integer) 
c     iday : 3-digit day (001-366) (integer)
c 
c  output parameters -
c     iyymmd : yymmdd (6-digit integer) 
c              (year*10000 + month*100 + day) 
c 
c  key local parameters - 
c     jday   : number of days into the year 
c     leap   : leap year indicator (0=leap year)
c     idymon : array of days elapsed as of the start of each month. 
c                first dimension is 1 for regular year, 2 for leap year.
c     irow   : first (leap year) dimensioning indicator for idymon
c     imon1  : second (month) dimensioning indicator for idymon 
c 
c  subroutines called - 
c     none
c  comments - 
c     this sub will not consider leap centuries 
c 
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
c 
c 
c*****
      parameter ( ianal = 1 ) 
c 
c$    if ( ianal .eq. 1 ) then
         common /utlcnt/ ncalls(60) 
c$    endif 
c*****
      dimension idymon(2,13)
c 
      data (idymon(1,k), k=1, 13) 
     2   /0,31,59,90,120,151,181,212,243,273,304,334,365/ 
      data (idymon(2,k), k=1, 13) 
     2   /0,31,60,91,121,152,182,213,244,274,305,335,366/ 
c 
c$    if ( ianal .eq. 1 ) then
         ncalls( 54 ) = ncalls( 54 ) + 1
c$    endif 
c 
      jday   = iday 
      leap   = iyr - ( iyr/4 )*4
c 
      if ( leap .eq. 0 ) then 
         irow = 2 
      else
         irow = 1 
      end if
c 
c *** locate the proper month of the year by stepping backwards 
c *** and checking total days elapsed.
c 
      imon1 = 13
c 
c *** do until (idymon .lt. jday) 
c 
 1380 imon1 = imon1 - 1 
      if ( .not. ( idymon(irow,imon1) .lt. jday ) ) goto 1380 
c 
c *** end do
c 
      jday = jday - idymon(irow,imon1)
      iyymmd = ( iyr*10000 ) + ( imon1*100 ) + jday 
c 
      return
      end 
